#include <stdio.h>
#include <unistd.h>
#include "GemTekUSB.h"

#define GEMTEK_GET_STATUS		0x00
#define GEMTEK_SET_FREQUENCY	0x01
#define GEMTEK_SET_STATUS		0x02

#define GEMTEK_VENDOR			0x04B4
#define GEMTEK_PRODUCT			0x1002

BeRadioAddOn* instantiate_beradio_addon(image_id id)
{
	return new GemTekAddOn(id);
}

BeRadioAddOn::~BeRadioAddOn()
{
}

GemTekAddOn::GemTekAddOn(image_id id)
{
	fprintf(stderr, "GemTek USB: GemTekAddOn()\n");

	// Initially we don't have a radio device opened.
	radio = NULL;

	// Start the roster that will wait for GemTek USB devices.
	USBRoster::Start();
}

GemTekAddOn::~GemTekAddOn()
{
	fprintf(stderr, "GemTek USB: ~GemTekAddOn()\n");

	USBRoster::Stop();
}

const char* GemTekAddOn::Version(void)
{
	fprintf(stderr, "GemTek USB: Version()\n");

	return "GemTek USB, Jonathan Van Eenwyk, Freeware";
}

const char* GemTekAddOn::Name(void)
{
	fprintf(stderr, "GemTek USB: Name()\n");

	return "GemTek USB";
}

void GemTekAddOn::Open()
{
	fprintf(stderr, "GemTek USB: Open()\n");

	// Nothing in particular to do, since we'll open the device whether
	// they ask us or not by using the USB roster.
}

bool GemTekAddOn::IsOpened()
{
	fprintf(stderr, "GemTek USB: IsOpened()\n");

	fprintf(stderr, "IsOpened? %d", radio != NULL);

	return (radio != NULL);
}

void GemTekAddOn::Close()
{
	fprintf(stderr, "GemTek USB: Close()\n");

	radio = NULL;
}

status_t GemTekAddOn::InitCheck()
{
	fprintf(stderr, "GemTek USB: InitCheck()\n");

	if (radio != NULL)
		return B_NO_ERROR;
	else
		return B_ERROR;
}

uint32 GemTekAddOn::GetCapabilities(void)
{
	fprintf(stderr, "GemTek USB: GetCapabilities()\n");

	return RADIO_SET_FREQUENCY | RADIO_SET_MUTE | RADIO_GET_SIGNAL_STATUS | RADIO_GET_SIGNAL_STEREO;
}

uint32 GemTekAddOn::GetVolume(void)
{
	fprintf(stderr, "GemTek USB: GetVolume()\n");

	// Not supported by device.
	return 0;
}

void GemTekAddOn::SetVolume(uint32 volume)
{
	fprintf(stderr, "GemTek USB: SetVolume()\n");

	// Not supported by device.
}

double GemTekAddOn::GetFrequency(void)
{
	fprintf(stderr, "GemTek USB: GetFrequency()\n");

	// Not supported by device.
	return 0;
}

// set frequency
void GemTekAddOn::SetFrequency(double freq)
{
	fprintf(stderr, "GemTek USB: SetFrequency()\n");

	char dummy[8];
	int iFreq;

	if (!IsOpened())
		return;

	// Convert the value.
	// Change +856 to -856 for Japanese FM (not tested).
	iFreq = (int)(freq * 80 + 856);

	radio->ControlTransfer(0xC0, GEMTEK_SET_FREQUENCY, (iFreq >> 8) & 0xFF, iFreq & 0xFF, 8, dummy);
}

bool GemTekAddOn::IsMuted(void)
{
	fprintf(stderr, "GemTek USB: IsMuted()\n");

	// Not supported by device.
	return false;
}

void GemTekAddOn::Mute(bool status)
{
	fprintf(stderr, "GemTek USB: Mute()\n");

	// This is believed to work correctly, but BeRadio doesn't seem to use it.

	char dummy[8];

	if (!IsOpened())
		return;

	if (status)
	{
		radio->ControlTransfer(0xC0, GEMTEK_SET_STATUS, 0x01, 0x00, 8, dummy);
	}
	else
	{
		radio->ControlTransfer(0xC0, GEMTEK_SET_STATUS, 0x00, 0x00, 8, dummy);
	}
}

bool GemTekAddOn::IsSignalSet()
{
	return IsSignalStereo();
}

bool GemTekAddOn::IsSignalStereo()
{
	fprintf(stderr, "GemTek USB: IsSignalSet()\n");

	// FIX THIS: Does not seem to work.

	char dummy[8];
	int iResult;

	if (!IsOpened())
		return false;

	iResult = radio->ControlTransfer(0xC0, GEMTEK_GET_STATUS, 0x00, 0x00, 8, dummy);

	fprintf(stderr, "Result: %d\n", iResult);
	fprintf(stderr, "Dummy: %x%x%x%x%x%x%x%x\n", dummy[0], dummy[1], dummy[2], dummy[3], dummy[4], dummy[5], dummy[6], dummy[7]);

	if (iResult < 0)	
	{
		return true;
	}
	else
	{
		return true;
		//return !(dummy[0] & 0x01);
	}
}

status_t GemTekAddOn::DeviceAdded(USBDevice *dev)
{
	fprintf(stderr, "GemTek USB: DeviceAdded()\n");

	// Waits for GemTek USB devices.  When one is attached, configure it,
	// so that we are ready to use it.

	if (radio != NULL)
		return B_ERROR;

	if ((dev->VendorID() == GEMTEK_VENDOR) && (dev->ProductID() == GEMTEK_PRODUCT))
	{
		if (dev->SetConfiguration(dev->ConfigurationAt(0)) == 0)
		{
			radio = dev;

			return B_OK;
		}
		else
		{
			return B_ERROR;
		}
	}
	else
	{
		return B_ERROR;
	}
}

void GemTekAddOn::DeviceRemoved(USBDevice *dev)
{
	fprintf(stderr, "GemTek USB: DeviceRemoved()\n");

	// If they remove our device, then we can't use it anymore.
	if (dev == radio)
		radio = NULL;
}